searchNodes=[{"type":"module","title":"Wafer","doc":"Welcome to Wafer. The funkiest way to write hardware drivers. This library doesn't do much on it's own, it is used to help with some of the repetitive tasks of writing drivers for hardware peripherals such as I2C and SPI connected sensors. Wafer works with both ElixirALE and Circuits. As such it's up to you to define which dependency you're using.","ref":"Wafer.html"},{"type":"protocol","title":"Wafer.Chip","doc":"A Chip is a physical peripheral with registers which can be read from and written to. Rather than interacting with this protocol directly, it's a lot easier to use the macros in Wafer.Registers to do it for you. Deriving If you're implementing your own Conn type which simply delegates to one of the lower level drivers then you can derive this protocol automatically: defmodule MyConnection do @derive Wafer.Chip defstruct [ :conn ] end If your type uses a key other than conn for the inner connection you can specify it while deriving: defmodule MyConnection do @derive { Wafer.Chip , key : :i2c_conn } defstruct [ :i2c_conn ] end A note on SPI devices You will have to manually implement this protocol for SPI devices as there is no conventional way to read and write registers over SPI and every device has their own way of implementing an SPI instruction set.","ref":"Wafer.Chip.html"},{"type":"function","title":"Wafer.Chip.read_register/3","doc":"Read the register at the specified address. Arguments conn a type which implements the Wafer.Conn behaviour. register_address the address of the register to read from. bytes the number of bytes to read from the register. Example iex&gt; { :ok , conn } = ElixirALE.I2C . acquire ( bus : &quot;i2c-1&quot; , address : 0x68 ) ...&gt; Chip . read_register ( conn , 0 , 1 ) { :ok , &lt;&lt; 0 &gt;&gt; }","ref":"Wafer.Chip.html#read_register/3"},{"type":"function","title":"Wafer.Chip.swap_register/3","doc":"Perform a swap with the register at the specified address. With some drivers this is atomic, and with others it is implemented as a register read followed by a write. Arguments conn a type which implements the Wafer.Conn behaviour. register_address the address of the register to swap. new_data the data to write to the regsiter. Returns The data that was previously in the register. Example iex&gt; { :ok , conn } = ElixirALE.I2C . acquire ( bus : &quot;i2c&quot; , address : 0x68 ) ...&gt; Chip . swap_register ( conn , 0 , &lt;&lt; 1 &gt;&gt; ) { :ok , &lt;&lt; 0 &gt;&gt; , conn }","ref":"Wafer.Chip.html#swap_register/3"},{"type":"function","title":"Wafer.Chip.write_register/3","doc":"Write to the register at the specified address. Arguments conn a type which implements the Wafer.Conn behaviour. register_address the address of the register to write to. data a bitstring or binary of data to write to the register. Example iex&gt; { :ok , conn } = ElixirALE.I2C . acquire ( bus : &quot;i2c&quot; , address : 0x68 ) ...&gt; Chip . write_register ( conn , 0 , &lt;&lt; 0 &gt;&gt; ) { :ok , conn }","ref":"Wafer.Chip.html#write_register/3"},{"type":"type","title":"Wafer.Chip.bytes/0","doc":"","ref":"Wafer.Chip.html#t:bytes/0"},{"type":"type","title":"Wafer.Chip.register_address/0","doc":"","ref":"Wafer.Chip.html#t:register_address/0"},{"type":"type","title":"Wafer.Chip.t/0","doc":"All the types that implement this protocol.","ref":"Wafer.Chip.html#t:t/0"},{"type":"behaviour","title":"Wafer.Conn","doc":"Defines a behaviour for connecting to a peripheral. This behaviour is used by all the driver types in Wafer and you should implement it for your devices also. Example Implementing Conn for a HTS221 chip connected via Circuits' I2C driver. defmodule HTS221 do defstruct ~w[conn]a alias Wafer.Driver.Circuits.I2C , as : Driver @behaviour Wafer.Conn @default_bus &quot;i2c-1&quot; @default_address 0x5F def acquire ( opts ) when is_list ( opts ) do bus = Keyword . get ( opts , :bus , @default_bus ) address = Keyword . get ( opts , :address , @default_address ) with { :ok , conn } &lt;- Driver . acquire ( bus_name : bus , address : address ) , do : { :ok , % HTS221 { conn : conn } } end end","ref":"Wafer.Conn.html"},{"type":"callback","title":"Wafer.Conn.acquire/1","doc":"Acquire a connection to a peripheral using the provided driver.","ref":"Wafer.Conn.html#c:acquire/1"},{"type":"type","title":"Wafer.Conn.option/0","doc":"","ref":"Wafer.Conn.html#t:option/0"},{"type":"type","title":"Wafer.Conn.options/0","doc":"","ref":"Wafer.Conn.html#t:options/0"},{"type":"type","title":"Wafer.Conn.t/0","doc":"","ref":"Wafer.Conn.html#t:t/0"},{"type":"module","title":"Wafer.DLL","doc":"Implements a safe bytewise data-link-layer which allows the transmission and receiption of arbitrary erlang terms in tiny packets. See Rx and Tx for details.","ref":"Wafer.DLL.html"},{"type":"module","title":"Wafer.DLL.Rx","doc":"Bytewise reception buffer for our data link layer.","ref":"Wafer.DLL.Rx.html"},{"type":"function","title":"Wafer.DLL.Rx.complete?/1","doc":"Has the reception completed successfully?","ref":"Wafer.DLL.Rx.html#complete?/1"},{"type":"function","title":"Wafer.DLL.Rx.error?/1","doc":"Was there an error during reception?","ref":"Wafer.DLL.Rx.html#error?/1"},{"type":"function","title":"Wafer.DLL.Rx.init/0","doc":"Initialize a new empty buffer reading for receiving.","ref":"Wafer.DLL.Rx.html#init/0"},{"type":"function","title":"Wafer.DLL.Rx.rx/2","doc":"Receive a byte into the buffer.","ref":"Wafer.DLL.Rx.html#rx/2"},{"type":"function","title":"Wafer.DLL.Rx.value/1","doc":"Attempt to retrieve the received value from the buffer.","ref":"Wafer.DLL.Rx.html#value/1"},{"type":"type","title":"Wafer.DLL.Rx.t/0","doc":"","ref":"Wafer.DLL.Rx.html#t:t/0"},{"type":"module","title":"Wafer.DLL.Tx","doc":"A bytewise transmission buffer for our data link layer.","ref":"Wafer.DLL.Tx.html"},{"type":"function","title":"Wafer.DLL.Tx.complete?/1","doc":"","ref":"Wafer.DLL.Tx.html#complete?/1"},{"type":"function","title":"Wafer.DLL.Tx.init/1","doc":"Initialize a new transmission buffer from a term.","ref":"Wafer.DLL.Tx.html#init/1"},{"type":"function","title":"Wafer.DLL.Tx.tx/1","doc":"Get the next byte to transmit.","ref":"Wafer.DLL.Tx.html#tx/1"},{"type":"type","title":"Wafer.DLL.Tx.t/0","doc":"","ref":"Wafer.DLL.Tx.html#t:t/0"},{"type":"protocol","title":"Wafer.DeviceID","doc":"Allows devices to generate a unique identifier. Mainly used for InterruptRegistry but you might find it useful too. Can be derived in the same manner as all other Wafer protocols.","ref":"Wafer.DeviceID.html"},{"type":"function","title":"Wafer.DeviceID.id/1","doc":"Generate a unique identifier for conn .","ref":"Wafer.DeviceID.html#id/1"},{"type":"type","title":"Wafer.DeviceID.t/0","doc":"All the types that implement this protocol.","ref":"Wafer.DeviceID.html#t:t/0"},{"type":"module","title":"Wafer.Driver.Circuits.GPIO","doc":"A connection to a native GPIO pin via Circuits' GPIO driver. Implements the Wafer.Conn behaviour as well as the Wafer.GPIO protocol.","ref":"Wafer.Driver.Circuits.GPIO.html"},{"type":"function","title":"Wafer.Driver.Circuits.GPIO.acquire/1","doc":"Acquire a connection to a native GPIO pin via Circuit's GPIO driver. Options :pin (required) the integer number of the pin to connect to. Hardware dependent. :direction (optional) either :in or :out . Defaults to :out .","ref":"Wafer.Driver.Circuits.GPIO.html#acquire/1"},{"type":"type","title":"Wafer.Driver.Circuits.GPIO.option/0","doc":"","ref":"Wafer.Driver.Circuits.GPIO.html#t:option/0"},{"type":"type","title":"Wafer.Driver.Circuits.GPIO.options/0","doc":"","ref":"Wafer.Driver.Circuits.GPIO.html#t:options/0"},{"type":"type","title":"Wafer.Driver.Circuits.GPIO.t/0","doc":"","ref":"Wafer.Driver.Circuits.GPIO.html#t:t/0"},{"type":"module","title":"Wafer.Driver.Circuits.GPIO.Dispatcher","doc":"This module implements a simple dispatcher for GPIO interrupts when using Circuits.GPIO . Because the Circuit's interrupt doesn't provide an indication of whether the pin is rising or falling we store the last known pin state and use it to compare.","ref":"Wafer.Driver.Circuits.GPIO.Dispatcher.html"},{"type":"function","title":"Wafer.Driver.Circuits.GPIO.Dispatcher.child_spec/1","doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"Wafer.Driver.Circuits.GPIO.Dispatcher.html#child_spec/1"},{"type":"function","title":"Wafer.Driver.Circuits.GPIO.Dispatcher.disable/2","doc":"Disable interrupts for this connection using the specified pin_condition.","ref":"Wafer.Driver.Circuits.GPIO.Dispatcher.html#disable/2"},{"type":"function","title":"Wafer.Driver.Circuits.GPIO.Dispatcher.enable/3","doc":"Enable interrupts for this connection using the specified pin_condition.","ref":"Wafer.Driver.Circuits.GPIO.Dispatcher.html#enable/3"},{"type":"module","title":"Wafer.Driver.Circuits.I2C","doc":"A connection to a chip via Circuits' I2C driver. Implements the Wafer.Conn behaviour as well as the Wafer.Chip and Wafer.I2C protocols.","ref":"Wafer.Driver.Circuits.I2C.html"},{"type":"function","title":"Wafer.Driver.Circuits.I2C.acquire/1","doc":"Acquire a connection to a peripheral using the Circuits' I2C driver on the specified bus and address.","ref":"Wafer.Driver.Circuits.I2C.html#acquire/1"},{"type":"type","title":"Wafer.Driver.Circuits.I2C.option/0","doc":"","ref":"Wafer.Driver.Circuits.I2C.html#t:option/0"},{"type":"type","title":"Wafer.Driver.Circuits.I2C.options/0","doc":"","ref":"Wafer.Driver.Circuits.I2C.html#t:options/0"},{"type":"type","title":"Wafer.Driver.Circuits.I2C.t/0","doc":"","ref":"Wafer.Driver.Circuits.I2C.html#t:t/0"},{"type":"module","title":"Wafer.Driver.Circuits.SPI","doc":"A connection to a chip via Circuits's SPI driver. Implements the Wafer.Conn behaviour as well as the Wafer.SPI protocol.","ref":"Wafer.Driver.Circuits.SPI.html"},{"type":"function","title":"Wafer.Driver.Circuits.SPI.acquire/1","doc":"Acquire a connection to a peripheral using the Circuits' SPI driver on the specified bus and address.","ref":"Wafer.Driver.Circuits.SPI.html#acquire/1"},{"type":"type","title":"Wafer.Driver.Circuits.SPI.driver_option/0","doc":"","ref":"Wafer.Driver.Circuits.SPI.html#t:driver_option/0"},{"type":"type","title":"Wafer.Driver.Circuits.SPI.option/0","doc":"","ref":"Wafer.Driver.Circuits.SPI.html#t:option/0"},{"type":"type","title":"Wafer.Driver.Circuits.SPI.options/0","doc":"","ref":"Wafer.Driver.Circuits.SPI.html#t:options/0"},{"type":"type","title":"Wafer.Driver.Circuits.SPI.t/0","doc":"","ref":"Wafer.Driver.Circuits.SPI.html#t:t/0"},{"type":"module","title":"Wafer.Driver.ElixirALE.GPIO","doc":"A connection to a native GPIO pin via ElixirALE's GPIO driver. Implements the Wafer.Conn behaviour as well as the Wafer.GPIO protocol.","ref":"Wafer.Driver.ElixirALE.GPIO.html"},{"type":"function","title":"Wafer.Driver.ElixirALE.GPIO.acquire/1","doc":"Acquire a connection to the specified GPIO pin using the ElixirALE GPIO driver. Options :pin (required) - the integer pin number. Hardware dependent. :direction - either :in or :out . Defaults to :out .","ref":"Wafer.Driver.ElixirALE.GPIO.html#acquire/1"},{"type":"type","title":"Wafer.Driver.ElixirALE.GPIO.option/0","doc":"","ref":"Wafer.Driver.ElixirALE.GPIO.html#t:option/0"},{"type":"type","title":"Wafer.Driver.ElixirALE.GPIO.options/0","doc":"","ref":"Wafer.Driver.ElixirALE.GPIO.html#t:options/0"},{"type":"type","title":"Wafer.Driver.ElixirALE.GPIO.t/0","doc":"","ref":"Wafer.Driver.ElixirALE.GPIO.html#t:t/0"},{"type":"module","title":"Wafer.Driver.ElixirALE.GPIO.Dispatcher","doc":"This module implements a simple dispatcher for GPIO interrupts when using ElixirALE .","ref":"Wafer.Driver.ElixirALE.GPIO.Dispatcher.html"},{"type":"function","title":"Wafer.Driver.ElixirALE.GPIO.Dispatcher.child_spec/1","doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"Wafer.Driver.ElixirALE.GPIO.Dispatcher.html#child_spec/1"},{"type":"function","title":"Wafer.Driver.ElixirALE.GPIO.Dispatcher.disable/2","doc":"Disable interrupts for this connection on the specified pin_condition.","ref":"Wafer.Driver.ElixirALE.GPIO.Dispatcher.html#disable/2"},{"type":"function","title":"Wafer.Driver.ElixirALE.GPIO.Dispatcher.enable/3","doc":"Enable interrupts for this connection using the specified pin_condition.","ref":"Wafer.Driver.ElixirALE.GPIO.Dispatcher.html#enable/3"},{"type":"module","title":"Wafer.Driver.ElixirALE.I2C","doc":"A connection to a chip via ElixirALE's I2C driver. Implements the Wafer.Conn behaviour as well as the Wafer.Chip and Wafer.I2C protocols.","ref":"Wafer.Driver.ElixirALE.I2C.html"},{"type":"function","title":"Wafer.Driver.ElixirALE.I2C.acquire/1","doc":"Acquire a connection to a peripheral using the ElixirALE I2C driver on the specified bus and address.","ref":"Wafer.Driver.ElixirALE.I2C.html#acquire/1"},{"type":"type","title":"Wafer.Driver.ElixirALE.I2C.option/0","doc":"","ref":"Wafer.Driver.ElixirALE.I2C.html#t:option/0"},{"type":"type","title":"Wafer.Driver.ElixirALE.I2C.options/0","doc":"","ref":"Wafer.Driver.ElixirALE.I2C.html#t:options/0"},{"type":"type","title":"Wafer.Driver.ElixirALE.I2C.t/0","doc":"","ref":"Wafer.Driver.ElixirALE.I2C.html#t:t/0"},{"type":"module","title":"Wafer.Driver.ElixirALE.SPI","doc":"A connection to a chip via ElixirALE's SPI driver. Implements the Wafer.Conn behaviour as well as the Wafer.SPI protocol.","ref":"Wafer.Driver.ElixirALE.SPI.html"},{"type":"function","title":"Wafer.Driver.ElixirALE.SPI.acquire/1","doc":"Acquire a connection to a peripheral using the ElixirALE' SPI driver on the specified bus and address.","ref":"Wafer.Driver.ElixirALE.SPI.html#acquire/1"},{"type":"type","title":"Wafer.Driver.ElixirALE.SPI.driver_option/0","doc":"","ref":"Wafer.Driver.ElixirALE.SPI.html#t:driver_option/0"},{"type":"type","title":"Wafer.Driver.ElixirALE.SPI.option/0","doc":"","ref":"Wafer.Driver.ElixirALE.SPI.html#t:option/0"},{"type":"type","title":"Wafer.Driver.ElixirALE.SPI.options/0","doc":"","ref":"Wafer.Driver.ElixirALE.SPI.html#t:options/0"},{"type":"type","title":"Wafer.Driver.ElixirALE.SPI.t/0","doc":"","ref":"Wafer.Driver.ElixirALE.SPI.html#t:t/0"},{"type":"module","title":"Wafer.Driver.Fake","doc":"This module provides a fake driver which you can use in your tests. It doesn't really do anything except return zeroes or empty results. I suggest that you use Mimic to set expectations for function calls and mock return values. This module implements the Chip , DeviceID , GPIO , I2C and SPI protocols.","ref":"Wafer.Driver.Fake.html"},{"type":"module","title":"Wafer.Format","doc":"Handy functions for formatting bytes, especially for debugging.","ref":"Wafer.Format.html"},{"type":"function","title":"Wafer.Format.to_bin/1","doc":"Convert the provided value into it's 0-padded byte-oriented string representation. Examples iex&gt; to_bin ( 0x1234 ) &quot;0b00010010_00110100&quot; iex&gt; to_bin ( &lt;&lt; 0xF0 , 0x0F &gt;&gt; ) &quot;0b11110000_00001111&quot;","ref":"Wafer.Format.html#to_bin/1"},{"type":"function","title":"Wafer.Format.to_hex/1","doc":"Convert the provided value into it's 0-padded byte-oriented string representation. Examples iex&gt; to_hex ( 0x1234 ) &quot;0x1234&quot; iex&gt; to_hex ( &lt;&lt; 0xF0 , 0x0F &gt;&gt; ) &quot;0xF00F&quot;","ref":"Wafer.Format.html#to_hex/1"},{"type":"protocol","title":"Wafer.GPIO","doc":"A GPIO is a physical pin which can be read from and written to. Some hardware supports interrupts, some has internal pull up/down resistors. Wafer supports all of these, however not all drivers do. Deriving If you're implementing your own Conn type that simply delegates to one of the lower level drivers then you can derive this protocol automatically: defstruct MyPin do @derive Wafer.GPIO defstruct [ :conn ] end If your type uses a key other than conn for the inner connection you can specify it while deriving: defstruct MyPin do @derive { Wafer.GPIO , key : :gpio_conn } defstruct [ :gpio_conn ] end","ref":"Wafer.GPIO.html"},{"type":"function","title":"Wafer.GPIO.direction/2","doc":"Set the pin direction.","ref":"Wafer.GPIO.html#direction/2"},{"type":"function","title":"Wafer.GPIO.disable_interrupt/2","doc":"Disables interrupts for this connection and pin_condition.","ref":"Wafer.GPIO.html#disable_interrupt/2"},{"type":"function","title":"Wafer.GPIO.enable_interrupt/3","doc":"Enable an interrupt for this connection and pin_condition. Arguments pin_condition - Either :rising or :falling . metadata - any optional data you wish to receive along with your interrupts. Defaults to nil . Interrupts will be sent to the calling process as messages in the form of {:interrupt, Conn.t(), pin_condition, metadata | nil} . Implementors note Wafer starts it's own Registry named Wafer.InterruptRegistry which you can use to publish your interrupts to using the above format. The registry key is set as follows: {PublishingModule, pin, pin_condition} . You can see examples in the Circuits.GPIO.Dispatcher and ElixirALE.GPIO.Dispatcher modules.","ref":"Wafer.GPIO.html#enable_interrupt/3"},{"type":"function","title":"Wafer.GPIO.pull_mode/2","doc":"Set the pull mode for this pin. If the hardware contains software-switchable pull-up and/or pull-down resistors you can configure them this way. If they are not supported then this function will return {:error, :not_supported} .","ref":"Wafer.GPIO.html#pull_mode/2"},{"type":"function","title":"Wafer.GPIO.read/1","doc":"Read the current pin value.","ref":"Wafer.GPIO.html#read/1"},{"type":"function","title":"Wafer.GPIO.write/2","doc":"Set the pin value.","ref":"Wafer.GPIO.html#write/2"},{"type":"type","title":"Wafer.GPIO.interrupt_option/0","doc":"","ref":"Wafer.GPIO.html#t:interrupt_option/0"},{"type":"type","title":"Wafer.GPIO.interrupt_options/0","doc":"","ref":"Wafer.GPIO.html#t:interrupt_options/0"},{"type":"type","title":"Wafer.GPIO.pin_condition/0","doc":"","ref":"Wafer.GPIO.html#t:pin_condition/0"},{"type":"type","title":"Wafer.GPIO.pin_direction/0","doc":"","ref":"Wafer.GPIO.html#t:pin_direction/0"},{"type":"type","title":"Wafer.GPIO.pin_number/0","doc":"","ref":"Wafer.GPIO.html#t:pin_number/0"},{"type":"type","title":"Wafer.GPIO.pin_value/0","doc":"","ref":"Wafer.GPIO.html#t:pin_value/0"},{"type":"type","title":"Wafer.GPIO.pull_mode/0","doc":"","ref":"Wafer.GPIO.html#t:pull_mode/0"},{"type":"type","title":"Wafer.GPIO.t/0","doc":"All the types that implement this protocol.","ref":"Wafer.GPIO.html#t:t/0"},{"type":"module","title":"Wafer.Guards","doc":"Handy guards which you can use in your code to assert correct values.","ref":"Wafer.Guards.html"},{"type":"macro","title":"Wafer.Guards.is_byte_size/1","doc":"Byte sizes are non negative integers","ref":"Wafer.Guards.html#is_byte_size/1"},{"type":"macro","title":"Wafer.Guards.is_i2c_address/1","doc":"An integer between 0 and 0x7F","ref":"Wafer.Guards.html#is_i2c_address/1"},{"type":"macro","title":"Wafer.Guards.is_pin_condition/1","doc":"One of :none , :rising , :falling or :both","ref":"Wafer.Guards.html#is_pin_condition/1"},{"type":"macro","title":"Wafer.Guards.is_pin_direction/1","doc":"Either :in or :out","ref":"Wafer.Guards.html#is_pin_direction/1"},{"type":"macro","title":"Wafer.Guards.is_pin_number/1","doc":"Pin numbers are non negative integers","ref":"Wafer.Guards.html#is_pin_number/1"},{"type":"macro","title":"Wafer.Guards.is_pin_pull_mode/1","doc":"One of :not_set , :none , :pull_up or :pull_down","ref":"Wafer.Guards.html#is_pin_pull_mode/1"},{"type":"macro","title":"Wafer.Guards.is_pin_value/1","doc":"Either 0 or 1","ref":"Wafer.Guards.html#is_pin_value/1"},{"type":"macro","title":"Wafer.Guards.is_register_address/1","doc":"Register addresses are non negative integers usually only one byte, but we don't enforce that here","ref":"Wafer.Guards.html#is_register_address/1"},{"type":"protocol","title":"Wafer.I2C","doc":"A protocol for interacting with I2C devices directly. Most of the time you'll want to use the Chip protocol for working with registers, but this is provided for consistency's sake. This API is extremely similar to the ElixirALE.I2C and Circuits.I2C APIs, except that it takes a Conn which implements I2C as an argument. Deriving If you're implementing your own Conn type that simply delegates to one of the lower level drivers then you can derive this protocol automatically: defstruct MyI2CDevice do @derive Wafer.I2C defstruct [ :conn ] end If your type uses a key other than conn for the inner connection you can specify it while deriving: defstruct MyI2CDevice do @derive { Wafer.I2C , key : :i2c_conn } defstruct [ :i2c_conn ] end","ref":"Wafer.I2C.html"},{"type":"function","title":"Wafer.I2C.detect_devices/1","doc":"Detect the devices adjacent to the connection's device on the same I2C bus.","ref":"Wafer.I2C.html#detect_devices/1"},{"type":"function","title":"Wafer.I2C.read/3","doc":"Initiate a read transaction to the connection's I2C device.","ref":"Wafer.I2C.html#read/3"},{"type":"function","title":"Wafer.I2C.write/3","doc":"Write data to the connection's I2C device.","ref":"Wafer.I2C.html#write/3"},{"type":"function","title":"Wafer.I2C.write_read/4","doc":"Write data to an I2C device and then immediately issue a read.","ref":"Wafer.I2C.html#write_read/4"},{"type":"type","title":"Wafer.I2C.address/0","doc":"","ref":"Wafer.I2C.html#t:address/0"},{"type":"type","title":"Wafer.I2C.data/0","doc":"","ref":"Wafer.I2C.html#t:data/0"},{"type":"type","title":"Wafer.I2C.option/0","doc":"","ref":"Wafer.I2C.html#t:option/0"},{"type":"type","title":"Wafer.I2C.options/0","doc":"","ref":"Wafer.I2C.html#t:options/0"},{"type":"type","title":"Wafer.I2C.t/0","doc":"All the types that implement this protocol.","ref":"Wafer.I2C.html#t:t/0"},{"type":"module","title":"Wafer.InterruptRegistry","doc":"This module provides Wafer's interrupt registry. This allows multiple subscribers to be subscribed to interrupts from many different pins. It is used by Driver.Circuits.GPIO.Dispatcher and Driver.ElixirALE.GPIO.Dispatcher and you should probably use it if you're writing your own driver which supports sending interrupts to subscribers.","ref":"Wafer.InterruptRegistry.html"},{"type":"function","title":"Wafer.InterruptRegistry.count_subscriptions/1","doc":"Count the number of active subscriptions for key .","ref":"Wafer.InterruptRegistry.html#count_subscriptions/1"},{"type":"function","title":"Wafer.InterruptRegistry.count_subscriptions/2","doc":"Count the number of active subscriptions for key and pin_condition .","ref":"Wafer.InterruptRegistry.html#count_subscriptions/2"},{"type":"function","title":"Wafer.InterruptRegistry.publish/2","doc":"Publish an interrupt to subscribers that are interested in key and pin_condition . Searches the registry for all subscribers to are subscribed to pin_condition or :both and publishes the interrupt message to them. The interrupt message takes the form {:interrupt, Conn.t(), :rising | :falling, metadata :: any} .","ref":"Wafer.InterruptRegistry.html#publish/2"},{"type":"function","title":"Wafer.InterruptRegistry.subscribe/3","doc":"Subscribe the calling process to interrupts for the specified pin_condition using the provided key . See subscribe/4 for more information. Example iex&gt; subscribe ( { MCP230017.Pin , 13 } , :rising , conn ) :ok","ref":"Wafer.InterruptRegistry.html#subscribe/3"},{"type":"function","title":"Wafer.InterruptRegistry.subscribe/4","doc":"Subscribe the calling process to interrupts for the specified pin_condition using the provided key . Arguments key a key which uniquely describes the pin. Probably a combination of device name and pin. pin_condition either :rising , :falling or :both . conn the receiver's connection to the pin, sent back to them in the interrupt message. metadata arbitrary data which will be sent to the receiver process in the interrupt message. Defaults to nil .","ref":"Wafer.InterruptRegistry.html#subscribe/4"},{"type":"function","title":"Wafer.InterruptRegistry.subscribers?/1","doc":"Are there any subscribers to key ?","ref":"Wafer.InterruptRegistry.html#subscribers?/1"},{"type":"function","title":"Wafer.InterruptRegistry.subscribers?/2","doc":"Are there any subscribers to key and pin_condition ?","ref":"Wafer.InterruptRegistry.html#subscribers?/2"},{"type":"function","title":"Wafer.InterruptRegistry.subscriptions/1","doc":"Returns a list of all subscriptions to key .","ref":"Wafer.InterruptRegistry.html#subscriptions/1"},{"type":"function","title":"Wafer.InterruptRegistry.subscriptions/2","doc":"Returns a list of all subscriptions key and pin_condition .","ref":"Wafer.InterruptRegistry.html#subscriptions/2"},{"type":"function","title":"Wafer.InterruptRegistry.unsubscribe/3","doc":"Remove all subscriptions for key , pin_condition and conn .","ref":"Wafer.InterruptRegistry.html#unsubscribe/3"},{"type":"type","title":"Wafer.InterruptRegistry.key/0","doc":"","ref":"Wafer.InterruptRegistry.html#t:key/0"},{"type":"module","title":"Wafer.Registers","doc":"This module provides helpful macros for specifying the registers used to communicate with your device. This can be a massive time saver, and means you can basically just copy them straight out of the datasheet. See the documentation for defregister/4 for more information.","ref":"Wafer.Registers.html"},{"type":"macro","title":"Wafer.Registers.defregister/3","doc":"Define functions for interacting with a device register with common defaults. Examples When specified with an access mode, assumes a 1 byte register: iex&gt; defregister ( :status , 0x03 , :ro ) When specified with a byte size, assumes a :rw register: iex&gt; defregister ( :config , 0x02 , 2 )","ref":"Wafer.Registers.html#defregister/3"},{"type":"macro","title":"Wafer.Registers.defregister/4","doc":"Define functions for interacting with a device register. Parameters name - name of the register. register_address - the address of the register. mode the access mode of the register. bytes the number of bytes in the register. Examples Read-only registers Define a read-only register named status at address 0x03 which is a single byte wide: iex&gt; defregister ( :status , 0x03 , :ro , 1 ) This will define the following function along with documentation and typespecs: def read_status ( conn ) , do : Chip . read_register ( conn , 0x03 , 1 ) Write-only registers Define a write-only register named int_en at address 0x02 which is 2 bytes wide: iex&gt; defregister ( :int_en , 0x02 , :wo , 2 ) This will define the following functions along with documentation and typespecs: def write_int_en ( conn , data ) when is_binary ( data ) and byte_size ( data ) == 2 , do : Chip . write_register ( conn , 0x2 , data ) def write_int_en ( _conn , data ) , do : { :error , &quot;Argument error: \#{ inspect ( data ) } &quot; } Read-write registers. Define a read-write register named config at address 0x01 . iex&gt; defregister ( :config , 0x01 , :rw , 1 ) In addition to defining read_config/1 and write_config/2 as per the examples above it will also generate the following functions along with documentation and typespecs: def swap_config ( conn , data ) when is_binary ( data ) and byte_size ( data ) == 1 , do : Chip . swap_register ( conn , 0x01 , data ) def swap_config ( _conn , data ) , do : { :error , &quot;Argument error: \#{ inspect ( data ) } &quot; } def update_config ( conn , callback ) when is_function ( callback , 1 ) do with { :ok , data } &lt;- Chip . read_register ( conn , 0x01 , 1 ) , new_data when is_binary ( new_data ) and byte_size ( new_data ) == 1 &lt;- callback . ( data ) , { :ok , conn } &lt;- Chip . write_regsiter ( conn , 0x01 , new_data ) , do : { :ok , conn } end def update_config ( _conn , _callback ) , do : { :error , &quot;Argument error: callback should be an arity 1 function&quot; }","ref":"Wafer.Registers.html#defregister/4"},{"type":"type","title":"Wafer.Registers.access_mode/0","doc":"","ref":"Wafer.Registers.html#t:access_mode/0"},{"type":"type","title":"Wafer.Registers.bytes/0","doc":"","ref":"Wafer.Registers.html#t:bytes/0"},{"type":"type","title":"Wafer.Registers.register_name/0","doc":"","ref":"Wafer.Registers.html#t:register_name/0"},{"type":"protocol","title":"Wafer.Release","doc":"A protocol for releasing connections. The opposite of Conn 's acquire/1 . Deriving If you're implementing your own Conn type that simply delegates to one of the lower level drivers that you can derive this protocol automatically: defstruct MyConn do @derive Wafer.Release defstruct [ :conn ] end If your type uses a key other than conn for the inner connection you can specify it while deriving: defstruct MyConn do @derive { Wafer.Release , key : :pin_conn } defstruct [ :pin_conn ] end","ref":"Wafer.Release.html"},{"type":"function","title":"Wafer.Release.release/1","doc":"Release all resources associated with the connection. Usually in preparation for shutdown.","ref":"Wafer.Release.html#release/1"},{"type":"type","title":"Wafer.Release.t/0","doc":"All the types that implement this protocol.","ref":"Wafer.Release.html#t:t/0"},{"type":"protocol","title":"Wafer.SPI","doc":"A (very simple) protocol for interacting with SPI connected devices. This API is a minimal version of the ElixirALE.SPI and Circuits.SPI APIs, except that it takes a Conn which implements SPI as an argument. If you want to use any advanced features, such as bus detection, I advise you to interact with the underlying driver directly. Deriving If you're implementing your own Conn type that simply delegates to one of the lower level drivers then you can derive this protocol automatically: defstruct MySPIDevice do @derive Wafer.SPI defstruct [ :conn ] end If your type uses a key other than conn for the inner connection you can specify it while deriving: defstruct MySPIDevice do @derive { Wafer.SPI , key : :spi_conn } defstruct [ :spi_conn ] end","ref":"Wafer.SPI.html"},{"type":"function","title":"Wafer.SPI.transfer/2","doc":"Perform an SPI transfer. SPI transfers are synchronous, so data should be a binary of bytes to send to the device, and you will receive back a binary of the same length containing the data received from the device.","ref":"Wafer.SPI.html#transfer/2"},{"type":"type","title":"Wafer.SPI.data/0","doc":"","ref":"Wafer.SPI.html#t:data/0"},{"type":"type","title":"Wafer.SPI.t/0","doc":"All the types that implement this protocol.","ref":"Wafer.SPI.html#t:t/0"},{"type":"module","title":"Wafer.Twiddles","doc":"Handy functions for dealing with bits and bytes in the wild.","ref":"Wafer.Twiddles.html"},{"type":"function","title":"Wafer.Twiddles.clear_bit/2","doc":"Set the specified bit to 0 within the byte . Example iex&gt; clear_bit ( 3 , 0 ) 2 iex&gt; clear_bit ( &lt;&lt; 3 &gt;&gt; , 0 ) &lt;&lt; 2 &gt;&gt;","ref":"Wafer.Twiddles.html#clear_bit/2"},{"type":"function","title":"Wafer.Twiddles.count_ones/1","doc":"Returns the number of 1's in byte . Example iex&gt; count_ones ( 0x7f ) 7 iex&gt; count_ones ( &lt;&lt; 0x7F &gt;&gt; ) 7","ref":"Wafer.Twiddles.html#count_ones/1"},{"type":"function","title":"Wafer.Twiddles.count_zeroes/1","doc":"Returns the number of 0's in byte . Example iex&gt; count_zeroes ( 0x7f ) 1 iex&gt; count_zeroes ( &lt;&lt; 0x7F &gt;&gt; ) 1","ref":"Wafer.Twiddles.html#count_zeroes/1"},{"type":"function","title":"Wafer.Twiddles.find_ones/1","doc":"Find all the 1 bits in byte and return a list of bit_number s. Example iex&gt; find_ones ( 0x0A ) [ 1 , 3 ] iex&gt; find_ones ( &lt;&lt; 0x0A &gt;&gt; ) [ 1 , 3 ]","ref":"Wafer.Twiddles.html#find_ones/1"},{"type":"function","title":"Wafer.Twiddles.find_zeroes/1","doc":"Find all the 0 bits in byte and return a list of bit_number s. Example iex&gt; find_zeroes ( 0xFA ) [ 0 , 2 ] iex&gt; find_zeroes ( &lt;&lt; 0xFA &gt;&gt; ) [ 0 , 2 ]","ref":"Wafer.Twiddles.html#find_zeroes/1"},{"type":"function","title":"Wafer.Twiddles.get_bit/2","doc":"Get the bit at the specified bit_number within byte . Example iex&gt; get_bit ( 0x7f , 6 ) 1 iex&gt; get_bit ( &lt;&lt; 0x7F &gt;&gt; , 6 ) 1","ref":"Wafer.Twiddles.html#get_bit/2"},{"type":"function","title":"Wafer.Twiddles.get_bool/2","doc":"Return a bit out of a binary as a boolean value. Example iex&gt; get_bool ( 0x7f , 6 ) true iex&gt; get_bool ( &lt;&lt; 0x7F &gt;&gt; , 6 ) true","ref":"Wafer.Twiddles.html#get_bool/2"},{"type":"macro","title":"Wafer.Twiddles.is_bit/1","doc":"","ref":"Wafer.Twiddles.html#is_bit/1"},{"type":"macro","title":"Wafer.Twiddles.is_bit_number/1","doc":"","ref":"Wafer.Twiddles.html#is_bit_number/1"},{"type":"macro","title":"Wafer.Twiddles.is_byte/1","doc":"","ref":"Wafer.Twiddles.html#is_byte/1"},{"type":"function","title":"Wafer.Twiddles.set_bit/2","doc":"Set a the specified bit_number to 1 within the byte . Example iex&gt; set_bit ( 0 , 0 ) 1 iex&gt; set_bit ( &lt;&lt; 0 &gt;&gt; , 0 ) &lt;&lt; 1 &gt;&gt;","ref":"Wafer.Twiddles.html#set_bit/2"},{"type":"function","title":"Wafer.Twiddles.set_bit/3","doc":"Set the bit at the specified bit_number within the byte to the bit value. Example iex&gt; set_bit ( 0 , 1 , 1 ) 2 iex&gt; set_bit ( &lt;&lt; 0 &gt;&gt; , 1 , 1 ) &lt;&lt; 2 &gt;&gt; iex&gt; set_bit ( 0 , 1 , true ) 2 iex&gt; set_bit ( &lt;&lt; 0 &gt;&gt; , 1 , true ) &lt;&lt; 2 &gt;&gt;","ref":"Wafer.Twiddles.html#set_bit/3"},{"type":"type","title":"Wafer.Twiddles.bit/0","doc":"","ref":"Wafer.Twiddles.html#t:bit/0"},{"type":"type","title":"Wafer.Twiddles.bit_number/0","doc":"","ref":"Wafer.Twiddles.html#t:bit_number/0"},{"type":"type","title":"Wafer.Twiddles.single_byte_binary/0","doc":"","ref":"Wafer.Twiddles.html#t:single_byte_binary/0"}]